@@ -1,7 +1,7 @@
-# $Id: RankCorrelation.pm,v 1.26 2007/03/24 17:50:59 gene Exp $
+# $Id: RankCorrelation.pm 802 2007-12-03 22:16:27Z gene $
 
 package Statistics::RankCorrelation;
-our $VERSION = '0.10';
+our $VERSION = '0.12';
 use strict;
 use warnings;
 use Carp;
@@ -9,44 +9,36 @@ use Carp;
 sub new {
     my $proto = shift;
     my $class = ref($proto) || $proto;
-
-    # Handle the unnamed single, pair and optional parameters.
-    my( $u, $v ) = ( shift, shift );
-    my %args = ();
-    # Two vectors and parameters given.
-    if( @_ ) {
-        %args = @_;
-    }
-    # Parameters given but no vectors.
-    elsif( $u =~ /^\D+$/ && $v ) {
-        $args{$u} = $v;
-    }
-    else {
-        # Empty object created.
-    }
-
-    my $self  = {
-        x_data => $u || [],
-        y_data => $v || [],
-        sorted => $args{sorted},
-    };
-
+    my $self = {};
     bless $self, $class;
-    $self->_init;
+    $self->_init(@_);
     return $self;
 }
 
 sub _init {
     my $self = shift;
 
-    # Automatically compute the statistical ranks if given data.
+    # Handle vector and named parameters.
+    while( my $arg = shift ) {
+        if( ref $arg eq 'ARRAY' ) {
+               if( !defined $self->x_data ) { $self->x_data( $arg ) }
+            elsif( !defined $self->y_data ) { $self->y_data( $arg ) }
+            # XXX Handle extra data. Thanks. Love, Gene.
+        }
+        elsif( !ref $arg ) {
+            my $v = shift;
+            $self->{$arg} = defined $v ? $v : $arg;
+        }
+    }
+
+    # Automatically compute the ranks if given data.
     if( $self->x_data && $self->y_data &&
         @{ $self->x_data } && @{ $self->y_data }
     ) {
-        # "Co-normalize" the vectors if they are unequal.
+        # "Co-normalize" the vectors if they are of unequal size.
         my( $x, $y ) = pad_vectors( $self->x_data, $self->y_data );
 
-        # "Co-sort" the vector pair by the first one.
+        # "Co-sort" the bivariate data set by the first one.
         ( $x, $y ) = co_sort( $x, $y ) if $self->{sorted};
 
         # Set the massaged data.
@@ -56,9 +48,13 @@ sub _init {
         # Set the size of the data vector.
         $self->size( scalar @{ $self->x_data } );
 
-        # Set the ranks of the vectors.
-        $self->x_rank( rank( $self->x_data ) );
-        $self->y_rank( rank( $self->y_data ) );
+        # Set the ranks and ties of the vectors.
+        ( $x, $y ) = rank( $self->x_data );
+        $self->x_rank( $x );
+        $self->x_ties( $y );
+        ( $x, $y ) = rank( $self->y_data );
+        $self->y_rank( $x );
+        $self->y_ties( $y );
     }
 }
 
@@ -92,57 +88,76 @@ sub y_rank {
     return $self->{y_rank};
 }
 
-sub spearman {
-# Return Spearman's rho correlation coefficient.
+sub x_ties {
     my $self = shift;
+    $self->{x_ties} = shift if @_;
+    return $self->{x_ties};
+}
 
-    # Initialize the squared rank difference sum.
-    my $sq_sum = 0;
+sub y_ties {
+    my $self = shift;
+    $self->{y_ties} = shift if @_;
+    return $self->{y_ties};
+}
 
-    # Compute the sum of the absolute difference of the squared ranks.
+sub spearman {
+    my $self = shift;
+    # Algorithm contributed by Jon Schutz <Jon.Schutz@youramigo.com>:
+    my($x_sum, $y_sum) = (0, 0);
+    $x_sum += $_ for @{$self->{x_rank}};
+    $y_sum += $_ for @{$self->{y_rank}};
+    my $n = $self->size;
+    my $x_mean = $x_sum / $n;
+    my $y_mean = $y_sum / $n;
+    # Compute the sum of the difference of the squared ranks.
+    my($x_sum2, $y_sum2, $xy_sum) = (0, 0, 0);
     for( 0 .. $self->size - 1 ) {
-        $sq_sum += (abs(
-            $self->{x_rank}[$_] - $self->{y_rank}[$_]
-        )) ** 2;
-#warn "$sq_sum\n += ( $self->{x_rank}[$_] - $self->{y_rank}[$_] ) ** 2";
+        $x_sum2 += ($self->{x_rank}[$_] - $x_mean) ** 2;
+        $y_sum2 += ($self->{y_rank}[$_] - $y_mean) ** 2;
+        $xy_sum += ($self->{x_rank}[$_] - $x_mean) * ($self->{y_rank}[$_] - $y_mean);
     }
-
-#     warn "1 - ( (6 * $sq_sum) / ( $self->size * (( $self->size ** 2 ) - 1))\n";
-    return 1 - ( (6 * $sq_sum) / ( $self->size * (( $self->size ** 2 ) - 1))
-    );
+    return 1 if $x_sum2 == 0 || $y_sum2 == 0;
+    return $xy_sum / sqrt($x_sum2 * $y_sum2);
 }
 
 
 sub rank {
     my $u = shift;
 
-    # Make a list of ranks for each datum.
-    my %rank;
-    push @{ $rank{ $u->[$_] } }, $_ for 0 .. @$u - 1;
+    # Make a list of tied ranks for each datum.
+    my %ties;
+    push @{ $ties{ $u->[$_] } }, $_ for 0 .. @$u - 1;
 
     my ($old, $cur) = (0, 0);
 
     # Set the averaged ranks.
     my @ranks;
-    for my $x (sort { $a <=> $b } keys %rank) {
+    for my $x (sort { $a <=> $b } keys %ties) {
         # Get the number of ties.
-        my $ties = @{ $rank{$x} };
+        my $ties = @{ $ties{$x} };
         $cur += $ties;
 
         if ($ties > 1) {
             # Average the tied data.
             my $average = $old + ($ties + 1) / 2;
-            $ranks[$_] = $average for @{ $rank{$x} };
+            $ranks[$_] = $average for @{ $ties{$x} };
         }
         else {
             # Add the single rank to the list of ranks.
-            $ranks[ $rank{$x}[0] ] = $cur;
+            $ranks[ $ties{$x}[0] ] = $cur;
         }
 
         $old = $cur;
     }
 
-    return \@ranks;
+    # Remove the non-tied ranks.
+    while( my( $k, $v ) = each %ties ) {
+        delete $ties{$k} unless @$v > 1;
+    }
+
+    # Return the ranks arrayref in a scalar context and include ties
+    # if called in a list context.
+    return wantarray ? (\@ranks, \%ties) : \@ranks;
 }
 
 sub co_sort {
@@ -182,9 +197,7 @@ sub csim {
     return $k / (@$m1 * @$m1);
 }
 
-sub pad_vectors {
-# Append zeros to either vector for all values in the other that do
-# not have a corresponding value.
+sub pad_vectors { # Append zeros to either vector for all values in the other that do not have a corresponding value.
     my ($u, $v) = @_;
 
     if (@$u > @$v) {
@@ -197,9 +210,7 @@ sub pad_vectors {
     return $u, $v;
 }
 
-sub correlation_matrix {
-# Build a square, binary matrix that represents "higher or lower"
-# value within the given vector.
+sub correlation_matrix { # Build a square, binary matrix that represents "higher or lower" value within the given vector.
     my $u = shift;
     my $c;
 
@@ -213,34 +224,40 @@ sub correlation_matrix {
     return $c;
 }
 
-sub kendall_tau {
-# Return Kendall's tau correlartion coefficient
+sub kendall { # Return Kendall's tau correlation coefficient
     my $self = shift;
 
-    # Initialize number of concordant and discordant values.
-    my $concordant = 0;
-    my $discordant = 0;
-
-    # Get a list of the order of the index of the sorted ranks of the
-    # first list.
-    my @list =
-    map $_->[0] + 1,
-        sort { $a->[1] <=> $b->[1] }
-            map [ $_, $self->{x_rank}[$_] ],
-                0 .. $self->size - 1;
-
-    # calculate number of concordant and number of discordant
-    while (my $index = shift @list) {
-        for (@list) {
-            $concordant++
-                if $self->{x_rank}[ $index - 1 ] < $self->{y_rank}[ $_ - 1 ];
-            $discordant++
-                if $self->{y_rank}[ $index - 1 ] > $self->{y_rank}[ $_ - 1 ] ;
+    # Calculate number of concordant and discordant pairs.
+    my( $concordant, $discordant ) = ( 0, 0 );
+    for my $i ( 0 .. $self->size - 1 ) {
+        for my $j ( $i + 1 .. $self->size - 1 ) {
+            my $x_sign = sign( $self->{x_data}[$j] - $self->{x_data}[$i] );
+            my $y_sign = sign( $self->{y_data}[$j] - $self->{y_data}[$i] );
+            if (not($x_sign and $y_sign)) {}
+            elsif ($x_sign == $y_sign) { $concordant++ }
+            else { $discordant++ }
         }
     }
 
-    return (2 * ($concordant - $discordant)) /
-           ($self->size * ($self->size - 1));
+    # Set the indirect relationship.
+    my $d = $self->size * ($self->size - 1) / 2;
+    if( keys %{ $self->x_ties } || keys %{ $self->y_ties } ) {
+        my $x = 0;
+        $x += @$_ * (@$_ - 1) for values %{ $self->x_ties };
+        $x = $d - $x / 2;
+        my $y = 0;
+        $y += @$_ * (@$_ - 1) for values %{ $self->y_ties };
+        $y = $d - $y / 2;
+        $d = sqrt($x * $y);
+    }
+
+    return ($concordant - $discordant) / $d;
+}
+
+sub sign {
+    my $x = shift;
+    return 0 if $x == 0;
+    return $x > 0 ? 1 : -1;
 }
 
 1;
@@ -260,26 +277,31 @@ Statistics::RankCorrelation - Compute the rank correlation between two vectors
 
   $c = Statistics::RankCorrelation->new( $x, $y, sorted => 1 );
 
+  $n = $c->spearman;
+  $t = $c->kendall;
+  $m = $c->csim;
+
   $s = $c->size;
   $xd = $c->x_data;
   $yd = $c->y_data;
   $xr = $c->x_rank;
   $yr = $c->y_rank;
-
-  $n = $c->spearman;
-  $m = $c->csim;
-  $t = $c->kendall_tau;
+  $xt = $c->x_ties;
+  $yt = $c->y_ties;
 
 =head1 DESCRIPTION
 
 This module computes rank correlation coefficient measures between two 
 sample vectors.
 
-Working examples may be found in the distribution C<eg/> directory and 
-the module test file.
+Examples can be found in the distribution C<eg/> directory and the test
+file.  The C<FUNCTIONS> section below has to use when computing sorted
+rank coefficients by hand.
 
-Also the C<HANDY FUNCTIONS> section below has some ..handy functions 
-to use when computing sorted rank coefficients by hand.
+=head2 * IMPORTANT NOTE *
+
+This module does not compute correct results for Kendall's Tau with
+tied ranks.  I am working on this and have failing tests to prove it.
 
 =head1 METHODS
 
@@ -294,6 +316,9 @@ initialized by computing the statistical ranks of the vectors.  If
 they are of different cardinality the shorter vector is first padded
 with trailing zeros.
 
+If the <C<sorted> flag is set, the bivariate data set is sorted by the
+first (B<x>) vector.
+
 =head2 x_data
 
   $c->x_data( $y );
@@ -308,28 +333,42 @@ Return and set the one dimensional array reference data.  This is the
   $x = $c->y_data;
 
 Return and set the one dimensional array reference data.  This vector
-is dependent on the unit vector.
+is dependent on the x vector.
 
 =head2 size
 
   $c->size( $s );
   $s = $c->size;
 
-Return and set the number of elements in the unit array.
+Return and set the number of array elements.
 
 =head2 x_rank
 
   $c->x_rank( $rx );
   $rx = $c->x_rank;
 
-Return and set the ranked "unit" data as an array reference.
+Return and set the ranks as an array reference.
 
 =head2 y_rank
 
   $ry = $c->y_rank;
   $c->y_rank( $ry );
 
-Return (and optionally set) the ranked data as array references.
+Return and set the ranks as an array reference.
+
+=head2 x_ties
+
+  $xt = $c->x_ties;
+  $c->x_ties( $xt );
+
+Return and set the ties as a hash reference.
+
+=head2 y_ties
+
+  $yt = $c->y_ties;
+  $c->y_ties( $yt );
+
+Return and set the ties as a hash reference.
 
 =head2 spearman
 
@@ -339,22 +378,24 @@ Spearman's rho rank-order correlation is a nonparametric measure of
 association based on the rank of the data values and is a special 
 case of the Pearson product-moment correlation.
 
-The formula is:
-
-      6 * sum( (Xi - Yi)^2 )
+      6 * sum( (xi - yi)^2 )
   1 - --------------------------
-           n (n^2 - 1)
+             n^3 - n
 
-Where C<X> and C<Y> are the two rank vectors and C<i> is an index 
-from one to C<n> number of samples.
+Where C<x> and C<y> are the two rank vectors and C<i> is an index 
+from one to B<n> number of samples.
 
-=head2 kendall_tau
+=head2 kendall
 
-  $t = $c->kendall_tau;
+  $t = $c->kendall;
 
-         4P
-  t = --------- - 1
-      n (n - 1)
+         c - d
+  t = -------------
+      n (n - 1) / 2
+
+Where B<c> and B<c> are the number of concordant and discordant pairs
+and B<n> is the number of samples.  If there are tied pairs, a
+different (more complicated) denominator is used.
 
 =head2 csim
 
@@ -373,14 +414,14 @@ This measure has been studied in musical contour analysis.
 
 =head2 rank
 
-  $ranks = rank( [ 1.0, 3.2, 2.1, 3.2, 3.2, 4.3 ] );
+  $v = [qw(1 3.2 2.1 3.2 3.2 4.3)];
+  $ranks = rank($v);
   # [1, 4, 2, 4, 4, 6]
+  my( $ranks, $ties ) = rank($v);
+  # [1, 4, 2, 4, 4, 6], { 1=>[], 3.2=>[]}
 
-Return an array reference of the ordinal ranks of the given data.
-
-Note that the data must be sorted as measurement pairs prior to 
-computing the statistical rank.  This is done automatically by the
-object initialization method.
+Return an list of an array reference of the ordinal ranks and a hash
+reference of the tied data.
 
 In the case of a tie in the data (identical values) the rank numbers
 are averaged.  An example will elucidate:
@@ -416,10 +457,11 @@ Return the correlation matrix for a single vector.
 This function builds a square, binary matrix that represents "higher 
 or lower" value within the vector itself.
 
-=head1 TO DO
+=head2 sign
 
-Figure out what "> 0.5 discrepency" that Vladimir Babenko
-E<lt>Vl_Babenko@softhome.netE<gt> saw.
+Return 0, 1 or -1 given a number.
+
+=head1 TO DO
 
 Handle any number of vectors instead of just two.
 
@@ -431,7 +473,7 @@ For the C<csim> method:
 
 L<http://www2.mdanderson.org/app/ilya/Publications/JNMRcontour.pdf>
 
-For the C<spearman> and C<kendall_tau> methods:
+For the C<spearman> and C<kendall> methods:
 
 L<http://mathworld.wolfram.com/SpearmanRankCorrelationCoefficient.html>
 
@@ -439,21 +481,20 @@ L<http://en.wikipedia.org/wiki/Kendall's_tau>
 
 =head1 THANK YOU
 
-Thomas Breslin E<lt>thomas@thep.lu.seE<gt> for unsorted C<rank> code.
+Thomas Breslin E<lt>thomas@thep.lu.seE<gt>,
+Jerome E<lt>jerome.hert@free.frE<gt>,
+Jon Schutz E<lt>Jon.Schutz@youramigo.comE<gt> and
+Andy Lee E<lt>yikes2000@yahoo.comE<gt>
 
-Jerome E<lt>jerome.hert@free.frE<gt> for Kendall's tau
-
-=head1 AUTHOR
+=head1 AUTHOR AND COPYRIGHT
 
 Gene Boggs E<lt>gene@cpan.orgE<gt>
 
-=head1 COPYRIGHT
-
-Copyright 2003, Gene Boggs
+Copyright 2009, Gene Boggs, All Rights Reserved.
 
 =head1 LICENSE
 
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself. 
+This program is free software; you can redistribute or modify it under
+the same terms as Perl itself.
 
 =cut
